/*
	Name: GetMovies
	Version: 1.3.1
	Description: 	This script returns the movies of the day from www.dagenstv.com, with
			filtering for channels and time. It is very bandwidth-friendly, since
			it only updates once a day, after 06.00 when dagenstv.com should have
			updated it's pages.
			Credit goes out to Daniel Wessln for tricky time comparison.
	Author: Fredrik Edberg
	E-mail: edberg@homemail.com
*/


// set this variable to false in order to remove movies past current time
// setting it to true disables the trimming of movies by time
var allowAllTimes = false;

// set this variable to false in order to remove movies not matching the
// the channels defined below.
// setting it to true disables the trimming of movies by channels
var allowAllChannels = false;

// tv channels, enter the channels as they are named on www.dagenstv.com
var channels = new Array("SVT1", "SVT2", "TV3", "TV4", "Kanal 5", "Viasat Action/Nature", "ZTV", "TCM");

// if another format of the displayed line in Samurize is desired,
// locate the function formatLines_(data) in the code below.
// the line to alter is clearly marked in the code of the function.


//---------------------- CODE BELOW THIS LINE ----------------------

// the name of the temp file.
// the file will be saved in \samdata\ subfolder of the system TMP folder.
var fileName = "DagensTV_Filmer.tmp";

// this message is returned if the system is offline or the remote site inaccessible.
var OfflineMsg = "Not available";

// create and set the temp-path.
var fso = new ActiveXObject("Scripting.FileSystemObject");
var savePath = fso.GetSpecialFolder(2) + "\\samdata\\";
if (!fso.FolderExists(savePath))
	fso.CreateFolder(savePath);


// main web-fetching function.
function PageFetch()
{
	var data;
	var fs = new ActiveXObject("Scripting.FileSystemObject");
	if (!fs.fileexists(savePath + fileName)) // check if the file exists.
	{
		data = formatHTML_(GetHTML_()); // and if not retrieve content
		WriteFile_(savePath + fileName, data);  // then create/save the temp file.
	}
	else
		if (TimeToUpdate_(savePath + fileName)) // check if it is time to update the file
		{
			data = formatHTML_(GetHTML_()); // and if its too old, then retrieve content again
			WriteFile_(savePath + fileName, data);  // then write/save to temp file.
		}
		else
			data = ReadFile_(savePath + fileName); // otherwise we just read the temp file contents.
	if (data.indexOf(OfflineMsg) >= 0)  //if the file had problems being downloaded
		DeleteFile_(savePath + fileName);  //delete the file so a fresh version is downloaded next time
	data = formatLines_(data);
	delete fs;
	return data;
}

// this function formats the raw html to only contain the interesting bits.
// it does not control the format of the displayed line in Samurize.
function formatHTML_(html)
{
	var i = 0;
	var data = "";
	var lines = new Array();
	var line;
	var Movie, Watch, Time, Channel;
	if (html.indexOf(OfflineMsg) >= 0)
		return OfflineMsg;
	while (true)
	{
 		Movie = GetText_(html, "<span class=\"articleheader\">", "</span><br>", 0, i+1);
 		if (Movie == "")
 			break;
 		Watch = GetText_(html, "<span class=\"articlechannel\">", "</span>", 0, i+1);
 		Time = Watch.substr(Watch.length-5);
 		Channel = Watch.substr(0, Watch.indexOf(":")-3);
		lines[i] = Time + " " + Channel + "| " + Movie + "%b";
 		i++;
 	}
	lines = lines.sort(timeSort_);
	for (line in lines)
		data += lines[line];
	return data;
}

// this function formats the data for display in Samurize.
function formatLines_(data)
{
	var lines = new Array();
	var line;
	var Time, Channel, Movie;
	var i = 0;
	if (data.indexOf(OfflineMsg) >= 0)
		return OfflineMsg;
	while (data.indexOf("%b") >= 0)
	{
		Time = data.substr(0, 5);
		data = data.substr(6);
		Channel = data.substring(0, data.indexOf("|"));
		data = data.substr(data.indexOf("|")+2);
		Movie = data.substring(0, data.indexOf("%b"));
		data = data.substr(data.indexOf("%b")+2);
		if (Valid_(Channel, Time))
 		{
	 		// OUTPUT LINE - alter the line below for a different output to Samurize.
 			lines[i] = Time + " " + Movie + " (" + Channel + ")" + "%b";  // this line needs to end with '+ "%b"' , minus the '-characters.
 			i++;
 		}
 	}
 	data = "";
 	for (line in lines)
		data += lines[line];
	return data;
}

function timeSort_(n1, n2)
{
	var n1h = n1.substr(0,2);
	var n1m = n1.substr(3,2);
	var n2h = n2.substr(0,2);
	var n2m = n2.substr(3,2);

	var n1t = (parseInt(n1m) + 60*parseInt(n1h) + 19*60) % (24*60);
	var n2t = (parseInt(n2m) + 60*parseInt(n2h) + 19*60) % (24*60);

	if (n1t > n2t)
		return 1;
	if (n1t < n2t)
		return -1;
	return 0;
}

// this function returns true if the channel exists in the defined
// channels-array, otherwise it returns false.
function Valid_(Channel, Time)
{
	var i = 0;
	var vChannel = allowAllChannels;
	var vTime = allowAllTimes;
	while (i < channels.length)
	{
		if (Channel.length == channels[i].length && Channel.indexOf(channels[i]) >= 0)
			vChannel = true;
		i++;
	}

	var now = new Date();
	var mTime = new Date();
	var now_ = (now.getMinutes() + 60*now.getHours() + 19*60) % (24*60);
	mTime.setHours(parseInt(Time.substr(0,2)), parseInt(Time.substr(3,2)), 0);
	var mTime_ = (mTime.getMinutes() + 60*mTime.getHours() + 19*60) % (24*60);

	if (mTime_ >= now_)
		vTime = true;
	delete now;
	delete mTime;
	if (vChannel && vTime)
		return true;
	return false;
}

// this function extracts the desired text from the html-data
function GetText_(data,start,end,offset,rep)
{
	var i = 0;
	while (i < rep)
	{
	 	if (data.indexOf(start) < 0)
			return "";
		else
			data = data.substr(data.indexOf(start)+start.length+offset);
	 	i++;
	}
	data = data.substr(0,data.indexOf(end));
  return data;
}

// this function returns true if the file is older than a set time
// it takes one argument which is the complete file path.
function TimeToUpdate_(file)
{
	var result = false;
	var fs = new ActiveXObject("Scripting.FileSystemObject");
	if (fs.fileexists(file))
	{
		var f = fs.GetFile(file);
		var filetime = new Date(f.DateLastModified);
		var now = new Date();
		var updateTime = new Date();
		updateTime.setHours(6,15,0);
		if(now > updateTime && updateTime > filetime)
			result = true;
		delete filetime;
		delete now;
		delete updateTime;
	}
	delete fs;
	return result;
}

// this function reads a file and returns its content as text.
// it takes one argument which is the complete file path.
// returns -1 if file doesnt exist.
function ReadFile_(file)
{
	var fs = new ActiveXObject("Scripting.FileSystemObject");
	if (fs.fileexists(file))
	{
		var f = fs.OpenTextFile(file,1);
		var data = f.readall();
		f.close();
	}
	else
		var data = -1;
	delete fs;
	return data;
}

// this function deletes a file.
// it takes oen argument which is the complete file path.
function DeleteFile_(file)
{
	var fs = new ActiveXObject("Scripting.FileSystemObject");
	fs.DeleteFile(file);
	delete fs;
}

// This function writes content to a text file.
// it takes two arguments the complete file path and the content to write.
// note that it overwrites the file everytime.
function WriteFile_(file, content)
{
	var fs = new ActiveXObject("Scripting.FileSystemObject");
	var f = fs.CreateTextFile(file,true);
	f.write(content);
	f.close();
	delete fs;
}

// this function downloads the page and returns it.
function GetHTML_()
{
	try
	{
	var objXMLHTTP = new ActiveXObject("MSXML2.ServerXMLHTTP"); // requires MSXML 3.0+
	objXMLHTTP.open("GET", "http://www.dagenstv.com/se/movieguide/", false);
	objXMLHTTP.send();
	var html = BinToText_(objXMLHTTP.responseBody, 35000);
	}
	catch(e)
	{
		return OfflineMsg;
	}
	finally
	{
		delete objXMLHTTP;
	}
  return html;
}

// this function converts binary data to text.
// it takes two arguments, the first is the data, the second is the datasize.
function BinToText_(varBinData, intDataSizeInBytes)
{
	var adFldLong = 0x00000080;
	var adVarChar = 200;
	var objRS = new ActiveXObject("ADODB.Recordset");

	objRS.Fields.Append("txt", adVarChar, intDataSizeInBytes, adFldLong);
	objRS.Open();

	objRS.AddNew();
	objRS.Fields("txt").AppendChunk(varBinData);
	var html = objRS("txt").Value;

	objRS.Close();
	delete objRS;
	return html;
}